#if !defined(CREATURESCONVERSATION_H)
#define CREATURESCONVERSATION_H
//-------------------------------------------------------------------
// A class that demonstrates how to perform DDE calls to the
// Creatures program.
//
// Compiled and tested under Visual C++ v4.2 but only uses Win32
// specific calls should should be relatively easy to use elsewhere.
//
// NB: I worked out how to use the DDE interface using trial and
//     error. I spent much time using DDESpy to analyse how the
//     cyberlife programs worked. If this software causes Creatures
//     to die in spectacular manners please understand that you are
//     using this at your own risk! That said, I've had no problems.
//
// Some notes:
//	 - Very little C++ is used. Mainly calls are made to the DDEML
//     library. This file is just to show how the DDE interface works.
//
//   - Almost zero error handling.
//
// Example Usage with Visual C++:
//  Create a dialog box using the class wizard.
//  Add #include "CreaturesConversation.h" to the top of header file.
//  Add 'Creatures::Session fSession' to the class definition
//  Add a button to the dialog and add code to the button as:
//    'MessageBox(fSession.executeMacro("dde: getb cnam"), "test", "MB_OK);'
//  Add CreaturesConversation.cpp to the project.
//  Compile, run creatures, select a norn and execute your program.
//  Press the button and the macro should run.
//
// Provided by John Booth and in the public domain. Use as you wish.
// jwbooth@mailcity.com
//-------------------------------------------------------------------
#include <ddeml.h>

namespace Creatures
{
	// Callback function used by DDE
	HDDEDATA CALLBACK SessionCallback(
	    UINT uType,	// transaction type
	    UINT uFmt,	// clipboard data format
	    HCONV hconv,	// handle to the conversation
	    HSZ hsz1,	// handle to a string
	    HSZ hsz2,	// handle to a string
	    HDDEDATA hdata,	// handle to a global memory object
	    DWORD dwData1,	// transaction-specific data
	    DWORD dwData2 	// transaction-specific data
	);

	// Exception class for a DDE error
	class DDEException
	{
	};

	// Starts a DDE Session with creatures and allows
	// execution of arbitary macros.
	class Session
	{
	private:
		HSZ fVivariumString;
		HSZ fMacroString;
		HSZ fBlankString;
		HSZ fBrainActivityString;
		HSZ fSessionString;
		HCONV fConversation;
		DWORD fDDEInstance;

	public:
		Session();
		~Session();

		// Execute CAOS macro
		char const* executeMacro(char const* macro);

		// Brain Activity
		char const* getBrainActivity();

	private:
		// Perform cleanup of memory allocations, etc
		void cleanup();

		// Initialise DDE
		void initialiseDDE();
		void uninitialiseDDE();

		// Create a DDE String
		HSZ createDDEString(char const* s);
		void freeDDEString(HSZ s);

		// Create a conversation
		HCONV connectConversation(HSZ service, HSZ topic);
		void disconnectConversation(HCONV conversation);

		// An error has occurred - throw an exception
		void handleError();
	};
}
#endif
